<?php
/*
 * Copyright (C) 2017		 Oscss-Shop       <support@oscss-shop.fr>.
 *
 * This program is free software; you can redistribute it and/or modifyion 2.0 (the "License");
 * it under the terms of the GNU General Public License as published bypliance with the License.
 * the Free Software Foundation; either version 3 of the License, or
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

require_once(DOL_DOCUMENT_ROOT.'/core/class/doleditor.class.php');

/**
 * Description of mustacheTrait
 *
 * @author Oscss Shop <support@oscss-shop.fr>
 */
trait CommonObjectView
{

	/**
		@fn implementMustache($directory)
		@brief 
		@param string $directory path of mustache file 
		@return @mustache object 
	*/
    function implementMustache($directory = null)
    {
        global $db;
        Mustache_Autoloader::register();

        if ($directory === null) {
            $directory = $this->element;
        }
        
        
        if(substr($directory,0,-1) !='/' )
			$directory.='/'; 
			
        $this->MustachePaths = array(
            array(
                'path' => $directory.'views/',
                'extension' => '.html'
            ),
            array(
                'path' => dol_buildpath('/framework/views/'),
                'extension' => '.html'
            ),
            array(
                'path' => $directory.'js/',
                'extension' => '.js'
            ),
        );

        $loader          = new Mustache_Loader_CascadingLoader();
        $partials_loader = new Mustache_Loader_CascadingLoader();
// print_r($this->MustachePaths); 
// exit; 

        foreach ($this->MustachePaths as $path) {
            $dir = $path['path'];
            if (is_dir($dir)) {
                $loader->addLoader(
                    new Mustache_Loader_FilesystemLoader($dir, array('extension' => $path['extension']))
                );
                
                
                
                if (is_dir($dir.'partials/')) {
// 					var_dump($dir.'partials/'); 
					
                    $partials_loader->addLoader(
                        new Mustache_Loader_FilesystemLoader($dir.'partials/', array('extension' => $path['extension']))
                    );
                }
            }
        }

//        $loader->addLoader(
//            new Mustache_Loader_StringLoader()
//        );
        $loader->addLoader(
            new Mustache_Loader_ArrayLoader()
        );

        $this->Mustache = new Mustache_Engine(array(
            'template_class_prefix' => '__Mustache_',
// 				'cache' => DOL_DATA_ROOT.'/framework/mustache/tmp',
// 				'cache_file_mode' => 0666, // Please, configure your umask instead of doing this :)
// 				'cache_lambda_templates' => false,
            //load or file.html in tpl or direct <html>
            'loader' => $loader,
            'partials_loader' => $partials_loader,
            'helpers' => array(
                '__' => function($text) {
                    global $langs;
                    return $langs->trans($text);
                },
                'session' => function($sess) {
                    if (isset($_SESSION[$sess])) {
                        return $_SESSION[$sess];
                    } else {
                        return '';
                    }
                },
                'GETPOST' => function($field) {
                    return GETPOST($field);
                }
            ),
            'escape' => function($value) {
//             var_dump($value); 
				if(empty($value) )
					return ''; 

                return htmlspecialchars( (string)$value, ENT_COMPAT, 'UTF-8');
            },
            // 			'charset' => CHARSET,
            'logger' => new Mustache_Logger_StreamLogger('php://stderr'),
//            'logger' => new Mustache_Logger_StreamLogger(dolibarr_get_const($db, "SYSLOG_FILE", 0)),
            'strict_callables' => true, //see https://github.com/bobthecow/mustache.php/wiki/Callable
            'pragmas' => [Mustache_Engine::PRAGMA_FILTERS],
            )
        );
        
//         print_r($this->Mustache); 
// exit; 
        return $this->Mustache; // for usage machin::implementMuctache();
    }

    /**
		@fn load_fiche_titre()
		@brief 
		@return string
    */
    function load_fiche_titre()
    {
        global $langs;
        $mesg = '';
        return load_fiche_titre($langs->trans($this->action.$this->element.'Titre'), $mesg, dol_buildpath($this->rootDir.'img/'.$this->element.'.png', 2), 1);
    }

    /**
		@fn llx_header()
		@brief 
		@return string
    */
    function llx_header()
    {
        global $langs;
        ob_start();
        llxHeader('', $langs->trans($this->action.$this->context.$this->element.'Titre', $this->getRef()), '', '', 0, 0, $this->arrayofjs, $this->arrayofcss);
        $ob = ob_get_clean();
        ob_end_clean();
        return $ob;
    }

	/**
		@fn llx_footer()
		@brief 
		@return string
    */
    function llx_footer()
    {
        ob_start();
        llxFooter();
        $ob = ob_get_clean();
        ob_end_clean();
        return $ob;
    }

    /**
		@fn dol_fiche_head()
		@brief call native dol_fiche_head function
		@return object $ob result of ob_start/ob_end
    */
    function dol_fiche_head()
    {
        global $langs;
        ob_start();
        if (!empty($this->context)) {
            $head  = $this->prepare_head();
            $titre = $langs->trans($this->element.'Type');
        } else {
            $head  = array();
            $titre = '';
        }
        
        
        dol_fiche_head($head, $this->context, $titre, -1, $this->element.'@'.$this->element);

        $ob = ob_get_clean();
        ob_end_clean();
        return $ob;
    }

    /**
		@fn dol_fiche_end()
		@brief call native dol_fiche_head function
		@return object $ob result of ob_start/ob_end
    */
    function dol_fiche_end()
    {
        ob_start();
        dol_fiche_end();
        $ob = ob_get_clean();
        ob_end_clean();
        return $ob;
    }

    /**
		@fn dol_banner()
		@brief call native dol_banner function
		@return object $ob result of ob_start/ob_end
    */
    function dol_banner()
    {
        global $langs;
        ob_start();
        $linkback = '<a href="'.dol_buildPath($this->rootDir.$this->List.'', 2).(!empty($socid) ? '?socid='.$socid : '').'">'.$langs->trans("BackToList").'</a>';

        $morehtmlref = '<div class="refidno">';

        // Ref bis
        if (!empty($this->ref_ext)) {
            $morehtmlref .= $this->Form->editfieldkey("RefExt", 'ref_ext', $this->ref_ext, $this, $user->rights->{$this->element}->creer, 'string', '', 0, 1);
            $morehtmlref .= $this->Form->editfieldval("RefExt", 'ref_ext', $this->ref_ext, $this, $user->rights->{$this->element}->creer, 'string', '', null, null, '', 1);
            $morehtmlref .= '<br>';
        }
        
        // Thirdparty
        if( is_object($this->thirdparty) ) 
			$morehtmlref .= $langs->trans('ThirdParty').' : '.$this->thirdparty->getNomUrl(1);
        // Project
        if (!empty($conf->projet->enabled)) {
            $langs->load("projects");
            $morehtmlref .= '<br>'.$langs->trans('Project').' ';
            if ($user->rights->{$this->element}->creer) {
                if ($action != 'classify') {
                    $morehtmlref .= '<a href="'.$_SERVER['PHP_SELF'].'?action=classify&amp;id='.$this->id.'">'.img_edit($langs->transnoentitiesnoconv('SetProject')).'</a> : ';
                    if ($action == 'classify') {
                        //$morehtmlref.=$this->Form->form_project($_SERVER['PHP_SELF'] . '?id=' . $this->id, $this->socid, $this->fk_project, 'projectid', 0, 0, 1, 1);
                        $morehtmlref .= '<form method="post" action="'.$_SERVER['PHP_SELF'].'?id='.$this->id.'">';
                        $morehtmlref .= '<input type="hidden" name="action" value="classin">';
                        $morehtmlref .= '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
                        $morehtmlref .= $formproject->select_projects($this->socid, $this->fk_project, 'projectid', $maxlength, 0, 1, 0, 1, 0, 0, '', 1);
                        $morehtmlref .= '<input type="submit" class="button valignmiddle" value="'.$langs->trans("Modify").'">';
                        $morehtmlref .= '</form>';
                    } else {
                        $morehtmlref .= $this->Form->form_project($_SERVER['PHP_SELF'].'?id='.$this->id, $this->socid, $this->fk_project, 'none', 0, 0, 0, 1);
                    }
                }
            } else {
                if (!empty($this->fk_project)) {
                    $proj        = new Project($db);
                    $proj->fetch($this->fk_project);
                    $morehtmlref .= '<a href="'.DOL_URL_ROOT.'/projet/card.php?id='.$this->fk_project.'" title="'.$langs->trans('ShowProject').'">';
                    $morehtmlref .= $proj->ref;
                    $morehtmlref .= '</a>';
                } else {
                    $morehtmlref .= '';
                }
            }
        }

        $morehtmlref       .= '</div>';
        $moreparam         = '&context='.$this->context;
        $this->morehtmlref = $morehtmlref;
        $this->moreparam   = $moreparam;

        dol_banner_tab($this, 'id', $linkback, 1, 'rowid', 'ref', $morehtmlref, $moreparam);
        $ob = ob_get_clean();
        ob_end_clean();
        return $ob;
    }

    
	/**
		@fn AskForm()
		@brief Called by dol_form_confirmation MVC function 
			product display modal box action 
			
			This function is called after Hook FormConfirm
			
		@return string or null 
	*/
	protected function AskForm($action=''){
		global $db, $conf, $langs, $mysoc, $soc, $hookmanager, $form, $user; 
	
		return 0;
	}
    /**
		@fn dol_formconfirm()
		@brief  
		@note convention 
			for action based on modalbox confirm, use ask_action for value uf action 
			after confirm , this page contain value of action confirm_action 
		@return 
    */
    public function dol_formconfirm()
    {
		global $db, $conf, $langs, $mysoc, $soc, $hookmanager, $form, $user; 
		
        // to be defined from child element
        
        $txt = ''; 
		$parameters=array();
		$reshook=$hookmanager->executeHooks('FormConfirm',$parameters,$this,$this->action);    // Note that $action and $object may have been modified by some hooks
		if ($reshook < 0)
			setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');

		if (empty($reshook) && substr($this->action, 0,3) =='ask')
		{
			$act = (substr($this->action, 4) ); 
			
			$txt =$this->AskForm($act);
			
			if(empty($txt))
				// action is prefixed by ask_
				switch($act){
					case 'deleteline'; 
						$txt = $form->formconfirm($_SERVER["PHP_SELF"]."?id=".$this->id."&lineid=".GETPOST('lineid'),$langs->trans("DeleteLine"),$langs->trans("ConfirmDeleteLine"),"confirm_deleteline",'',0,1);
					break;
					
// 					case 'delete'; 
// 						$txt = $form->formconfirm($_SERVER['PHP_SELF']."?id=".$this->id,$langs->trans("Delete"),$langs->trans("ConfirmDelete"),"confirm_delete",'',0,1);
// 					break;
					
// 					case 'valid'; 
// 						$txt = $form->formconfirm($_SERVER['PHP_SELF']."?id=".$this->id,$langs->trans("Valide"),$langs->trans("ConfirmValide"),"confirm_valide",'',0,1);
// 					break;
					
					default: 
						$txt = $form->formconfirm($_SERVER['PHP_SELF']."?id=".$this->id,$langs->trans(ucwords($act)),$langs->trans("Confirm".ucwords($act)),"confirm_".$act,'',0,1);
				}
		}
		else
			return $hookmanager->resPrint; 


		return $txt; 
    }
    
    /**
		@fn dol_children()
		@brief 
		@return 
    */
    function dol_children()
    {
        // to be defined from child element
        // sorry first example was not havin child
    }

    
    /**
		@fn html_input()
		@brief 
		@return string
    */
    function html_input($name = 'input', $value = '', $placeholder = '', $values = array(), $notnull = false)
    {
        global $langs;
        // restrected values if set
        if (is_array($values) && count($values) && !in_array($value, $values)) {
            $value = '';
        }
        return '<input type="text" name="'.$name.'" class="flat mvc" placeholder="'.$langs->trans($placeholder).'" value="'.$value.'" />';
        
        
    }

    /**
		@fn html_textarea()
		@brief 
		@return string
    */
    function html_textarea($name = 'input', $value = '', $placeholder = '', $values = array(), $notnull = false)
    {
        global $langs, $conf;
        $doleditor = new DolEditor($name, $value, '100%', 300, 'full', '', true, true, $conf->global->FCKEDITOR_ENABLE_SOCIETE, 10, 80);
        return $doleditor->Create(1);
    }

    
    
    

    
    /**
		@fn html_select_company()
		@brief 
		@return string
    */
//     function html_datetime($name = 'input', $value = '', $placeholder = '', $values = array(), $notnull = false)
//     {
// 		ob_start();
// // 		print '<div>'; 
// 		
// // 		($set_time='', $prefix='re', $h=0, $m=0, $empty=0, $form_name="", $d=1, $addnowlink=0, $nooutput=0, $disabled=0, $fullday='', $addplusone='', $adddateof='')
// 		
//          $ob =$this->Form->select_date($value,$name,0,0,0,'',1,1,1);
//          
// //          var_dump($ob); exit; 
// //          
// //         print '</div>'; 
//         $ob = ob_get_contents();
//         
// // ob_end_clean();
//         ob_end_flush();
//         
// //                  var_dump($ob); exit; 
//         return $ob;
//     }
    
    
    /**
		@fn html_select()
		@brief 
		@return string
    */
    function html_select($name = 'input', $value = '', $placeholder = '', $values = array(), $notnull = false)
    {
        $showEmpty = ($notnull) ? 0 : 1;
        return $this->Form->selectarray($name, $values, $value, $showEmpty, 0, 0, '', 1);
    }

    /**
		@fn html_select_company()
		@brief 
		@return string
    */
    function html_select_company($name = 'input', $value = '', $placeholder = '', $values = array(), $notnull = false)
    {
        return $this->Form->select_company($value, $name);
    }

    /**
		@fn html_select_company()
		@brief 
		@return string
    */
    function html_select_product($name = 'input', $value = '', $placeholder = '', $values = array(), $notnull = false)
    {
		ob_start();
		print '<div>'; 
         $this->Form->select_produits($value, $name);
        print '</div>'; 
        $ob = ob_get_contents();
        ob_end_clean();
        return $ob;
    }
    
	 /**
		@fn html_select_doluser()
		@brief 
		@return string
    */
    function html_select_doluser($name = 'input', $value = '', $placeholder = '', $values = array(), $notnull = false)
    {
        $showEmpty = ($notnull) ? 0 : 1;
        return $this->Form->select_dolusers($value,$name,$showEmpty,'');
    }

	
}